# -*- coding: utf-8 -*-

from __future__ import print_function
from datetime import datetime
import numpy as np

from keystone.universe import KSUniverse as Universe
from keystone.parameters import DEFAULT_STARTING_CASH

class KSPosition(object):
    def __init__(self, security, quantity, price, commission):
        self.__security = security
        self.__quantity = quantity
        self.__priceArray = [price]
        self.__quantityArray = [quantity]
        self.__commissionArray = [commission]
        self.__datetimeArray = [Universe.time]
        self.__openAt = Universe.time
        self.__valueArray = [self.value()]
    
    def update(self, quantity, price, commission):
        self.__quantity += quantity
        self.__priceArray.append(price)
        self.__commissionArray.append(commission)
        self.__datetimeArray.append(Universe.time)
    
    def to_dict(self):
        history = []
        for commission, price, quantity, dt, value in zip(
            self.__commissionArray,
            self.__priceArray, 
            self.__quantityArray, 
            self.__datetimeArray,
            self.__valueArray):
            history.append({
                'datetime': dt,
                'quantity': quantity,
                'price': price,
                'commission': commission,
                'value': value
                })

        return {'sid': self.__security,
                'quantity': self.__quantity,
                'value': self.value(),
                'open_at': self.__openAt,
                'avg_cost': self.costBasis(),
                'total_commission': self.totalCommission(),
                'avg_commission': self.avgCommission(),
                'history': history}

    def __repr__(self):
        return self.to_dict().__repr__()
    
    def __str__(self):
        return self.to_dict().__str__()

    def isShort(self):
        return self.__quantity < 0
    
    def costBasis(self):
        return np.mean(self.__priceArray)
    
    def avgCommission(self):
        return np.mean(self.__commissionArray)

    def totalCommission(self):
        return np.sum(self.__commissionArray)

    def security(self):
        return self.__security
    
    
    def quantity(self):
        return self.__quantity
    
    
    def value(self):
        price = Universe.getPrice(self.__security)
        assert price is not None
        assert not np.isnan(price)
        return self.__quantity * price
    
    
    def openAt(self):
        return self.__openAt
    

class KSPortfolio(object):
    '''
    portfolio
    '''
    __startingCash = DEFAULT_STARTING_CASH
    __cash = DEFAULT_STARTING_CASH
    __positions = {}

    def __init__(self):
        pass
    
    @classmethod
    def update(self, security, quantity, price, commission):
        if security in self.__positions:
            self.__positions[security].update(quantity, price, commission)
            if self.__positions[security].quantity() == 0:
                self.__positions.pop(security)
        else:
            self.__positions[security] = KSPosition(security, quantity, price, commission)
        
        # TODO: 做空时计算方法要变
        self.__cash = self.__cash - (price * quantity) - commission
        
        # assert self.__cash >= 0.0
    
    @classmethod
    def startingCash(self):
        return self.__startingCash
        

    @classmethod
    def cash(self):
        return self.__cash
    
    
    @classmethod
    def securities(self):
        return list(self.__positions.keys())
    
    
    @classmethod
    def hasPosition(self, security):
        return security in self.__positions
    
    
    @classmethod
    def getPosition(self, security):
        if security in self.__positions:
            return self.__positions[security]
        else:
            raise KeyError("no such position for '" + security + u"'.")
      
    @classmethod      
    def positions(self):
        return list(self.__positions.values())
       
    @classmethod 
    def value(self):
        return self.__cash + np.sum([pos.value() for pos in list(self.__positions.values())])
    


# -*- coding: utf-8 -*-

class Riskless(object):
	value = 0.0314
	
	def to_dict(self):
		fields = copy(self.__dict__)
		return fields

	def __repr__(self):
		return self.to_dict().__repr__()

	def __str__(self):
		return self.to_dict().__str__()



# -*- coding: utf-8 -*-

from __future__ import print_function
import inspect
from six import with_metaclass
import abc
import numpy as np
from datetime import datetime

import keystone.parameters
from keystone.universe import KSUniverse as Universe

from keystone.utils import math_utils
from keystone.api import keystone_class, api_method
from keystone.coordinator import KSObserver, KSCoordinator
from keystone.sources.csv_source import KSCsvSource, KSMemorySource
from keystone.portfolio import KSPortfolio
from keystone.broker import KSBacktestingBroker, KSBrokerPolicy, KSDefaultBrokerPolicy
from keystone.action import KSContext, KSAction
from keystone.order import KSOrderEventType
from keystone.history import KSHistory
from keystone.exception import KSStopLineReachedException, KSEndTimeReachedException
from keystone.performance.perf import KSPerformance, PeriodReturn
from keystone.performance.analyzer import KSAnalyzer
from keystone.performance.sample_rate import KSSampleRate
from keystone.riskless import Riskless
from keystone.drain import KSDrain
from keystone.app_client.datetime_breakpoint import DatetimeBreakpoint
from keystone.py3compat import PY3, builtin_mod, iteritems, unicode_type, string_types
    
class KSStrategy(with_metaclass(abc.ABCMeta)):
    '''
    User strategy base class
    DO NOT use this class directly.

    _datetimeBreakPointCls is used by KPdb to set datetime breakpoints
    '''
    _datetimeBreakPointCls = DatetimeBreakpoint

    @abc.abstractmethod
    def onData(self, data, context, action):
        pass
    
    def onBeforeData(self, data, context, action):
        pass
    
    def onOrderEvent(self, orderEvent):
        pass

class EmptyStrategy(KSStrategy):
    def onData(self, data, context, action):
        pass

class KSStrategyManager(KSObserver):
    '''
    strategy Manager
    '''
    def __init__(self, 
                 strategyClass = EmptyStrategy,
                 broker = None,
                 history = None,
                 analyzers = None,
                 startTime = datetime.min,
                 endTime = datetime.max,
                 stopLossPercentage = 0.0):
        KSObserver.__init__(self)
        self.strategy = strategyClass()
        self.broker = broker
        self.history = history
        self.analyzers = analyzers
        self.startTime = startTime
        self.endTime = endTime
        self.stopLossPercentage = 0.0
        self.perf = KSPerformance()
        self.portfolio = KSPortfolio

        self.context = KSContext(self.history, self.analyzers)
        self.action = KSAction(self.broker)
    
    def onDataEvent(self, dataEvent):
        if self.strategy is None:
            return
        
        if dataEvent.time() < self.startTime:
            self.strategy.onBeforeData(dataEvent, self.context, self.action)
        elif dataEvent.time() > self.endTime:
            raise KSEndTimeReachedException()
        else:
            self.strategy.onData(dataEvent, self.context, self.action)
            # update performance
            self.perf.update(dataEvent, self.context)
            # update analyzer
            for analyzer in self.analyzers:
                analyzer.update(dataEvent, self.context)
        
        if self.portfolio.value() / self.portfolio.startingCash() < self.stopLossPercentage:
            raise KSStopLineReachedException()
        
    def onOrderEvent(self, orderEvent):
        if orderEvent.type != KSOrderEventType.ACCEPTED and orderEvent.type != KSOrderEventType.CANCELLED:
            self.updatePortfolio(orderEvent)
            
        if self.strategy is not None:
            self.strategy.onOrderEvent(orderEvent)
    
    def updatePortfolio(self, orderEvent):
        txn = orderEvent.txn
        self.portfolio.update(txn.sid, txn.quantity, txn.price, txn.commission)
        
    
class KSStrategyRunner(object):
    '''
    Strategy Runner
    '''
    def __init__(self):
        self.strategyClass = None
        self.analyzers = []
        self.history = None
        self.startingCash = keystone.parameters.DEFAULT_STARTING_CASH
        self.startTime = datetime.min
        self.endTime = datetime.max
        self.stopLossPercentage = keystone.parameters.DEFAULT_STOPLOSS_PERENTAGE
        self.drainAddress = None
        self.synAddress = None
    
    
    def setStrategy(self, strategy):
        if not issubclass(strategy, KSStrategy):
            raise TypeError("strategy必须为继承自'keystone.strategy.KSStrategy'的子类。")
        
        self.strategyClass = strategy
    def addSource(self, source):
        if not isinstance(source, KSCsvSource) and not isinstance(source, KSMemorySource):
            raise TypeError("source必须为'KSCsvSource'或'KSMemorySource'对象。")
        KSCoordinator.addSource(source)
    
    def setCash(self, cash):
        if not math_utils.isnumber(cash):
            raise TypeError("cash必须为数字。")
        self.startingCash = cash
        
    
    def setStartTime(self, startTime):
        if not isinstance(startTime, datetime):
            raise TypeError("startTime必须为'datetime'类型。")
        
        if self.endTime < startTime:
            raise TypeError("startTime不可大于endTime。")
        
        self.startTime = startTime
        
    
    def setEndTime(self, endTime):
        if not isinstance(endTime, datetime):
            raise TypeError("endTime必须为'datetime'类型。")
        
        if endTime < self.startTime:
            raise TypeError("endTime不可小于startTime。")
        
        self.endTime = endTime
        
    def setStopPercentage(self, percentage):
        if not math_utils.isnumber(percentage) or percentage < 0.0 or percentage > 1.0:
            raise TypeError("percentage必须满足 0 <= percentage <= 1")
        
        self.stopLossPercentage = percentage
        
    
    def attachAnalyzer(self, analyzer):
        if not isinstance(analyzer, KSAnalyzer):
            raise TypeError("analyzer必须为继承自'keystone.performance.analyzer.KSAnalyzer'的子类实例对象。")
        self.analyzers.append(analyzer)
    
    
    def setHistoryCapacity(self, capacity):
        if not math_utils.isnumber(capacity):
            raise TypeError("capacity必须为整数。")
        self.history = KSHistory(capacity)
        
    def setDrainAddress(self, address, synAddress = None):
        self.drainAddress = address
        self.synAddress = synAddress

    def setBenchmark(self, column, sid=[]):
        if sid != [] and not isinstance(sid, string_types):
            raise TypeError("sid必须为字符串。")
        if not isinstance(column, string_types):
            raise TypeError("column必须为字符串。")
        PeriodReturn.setBenchmark(sid, column)

    def setSampleRate(self, sampleRate):
        if sampleRate not in [KSSampleRate.MONTH, KSSampleRate.DAY, KSSampleRate.MINUTE, KSSampleRate.HOUR, KSSampleRate.SECOND]:
            raise TypeError("sampleRate格式错误。")
        KSSampleRate.rate = sampleRate

    def setTradingDays(self, days):
        if not math_utils.isint(days) or days < 1 or dasy > 360:
            raise TypeError("days必须为[1,360]之间的整数。")
        KSSampleRate.tradingDays = days

    def setTradingHours(self, hours):
        if not math_utils.isnumber(hours):
            raise TypeError("hours必须为整数。")
        KSSampleRate.tradingHours = hours

    def setRiskless(self, riskless):
        if not math_utils.isnumber(riskless):
            raise TypeError("riskless必须为整数。")
        Riskless.value = riskless

    
    def setBrokerPolicy(self, brokerPolicy):
        if not isinstance(brokerPolicy, KSBrokerPolicy):
            raise TypeError("brokerPolicy必须为继承自'keystone.broker.KSBrokerPolicy'的子类实例对象.")
        KSBacktestingBroker.brokerPolicy = brokerPolicy

    def setInstantMatch(self):
        KSBacktestingBroker.brokerPolicy.turnOnInstantMatch()

    def setSlippage(self, model):
        KSBacktestingBroker.brokerPolicy.setSlippageModel(model)

    def setCommission(self, model):
        KSBacktestingBroker.brokerPolicy.setCommissionModel(model)

    def run(self):
        if self.strategyClass is None:
            raise ValueError("can not find user strategy")

        # Initialize Backtesting Broker
        broker = KSBacktestingBroker()

        # Initialize Strategy Manager
        strategyManager = KSStrategyManager(strategyClass = self.strategyClass,
                                            broker = broker,
                                            history = self.history,
                                            analyzers = self.analyzers,
                                            startTime = self.startTime,
                                            endTime = self.endTime,
                                            stopLossPercentage = self.stopLossPercentage)
        
        # Initialize drain
        self.drain = None
        if self.drainAddress is not None:
            self.drain = KSDrain(strategyManager, drain_url=self.drainAddress, syn_url=self.synAddress)
            self.drain.start()

        # add data observer, order matters
        KSCoordinator.addDataObserver(Universe)
        KSCoordinator.addDataObserver(broker)
        KSCoordinator.addDataObserver(strategyManager)
        if self.history is not None:
            KSCoordinator.addDataObserver(self.history)
        if self.drain is not None:
            KSCoordinator.addDataObserver(self.drain)
        
        # add order observer
        KSCoordinator.addOrderObserver(strategyManager)
        if self.drain is not None:
            KSCoordinator.addOrderObserver(self.drain)
        # run
        try:
            KSCoordinator.run()
        except KSEndTimeReachedException:
            pass
        except KSStopLineReachedException:
            print("已达止损线，回测将自动停止。")
            
        if self.drain is not None:
            self.drain.wait()



# -*- coding: utf-8 -*-

from __future__ import print_function
from six import with_metaclass
import abc
import numpy as np
from datetime import datetime

from keystone.api import keystone_class, api_method
from keystone.utils import math_utils

class KSCommissionModel(with_metaclass(abc.ABCMeta)):
    def __init__(self):
        pass
    
    @abc.abstractmethod
    def compute(self, dataEvent, order, *args, **kwargs):
        '''
        INPUT:
            dataEvent - Current data event
            order - KSOrder instance about the current order
            args - User custom args
            kwargs - User custom kwargs
        OUTPUT:
            user MUST return a commission
        '''
        return 0
    
class KSPerShareCommissionModel(KSCommissionModel):
    def __init__(self, rate):
        KSCommissionModel.__init__(self)
        if not math_utils.isnumber(rate):
            raise TypeError("rate必须为数字。")
        self.rate = rate
        
    def compute(self, dataEvent, order, *args, **kwargs):
        shares = order.remaining()
        return abs(shares) * self.rate
    
class KSTransactionValueCommissionModel(KSCommissionModel):
    def __init__(self, rate):
        KSCommissionModel.__init__(self)
        if not math_utils.isnumber(rate):
            raise TypeError("rate必须为数字。")
        self.rate = rate
    
    def compute(self, dataEvent, order, *args, **kwargs):
        shares = order.remaining()
        price = dataEvent.query(sid, 'price')
        return abs(shares * price * self.rate)




# -*- coding: utf-8 -*-

from copy import copy
from six import with_metaclass
import abc
import numpy as np
from datetime import datetime, timedelta
from collections import deque

from keystone.performance import risk
from keystone.performance.analyzer import KSAnalyzer
from keystone.performance.sample_rate import KSSampleRate
from keystone.parameters import DEFAULT_BENCHMARK_COLUMN

class PeriodReturn(object):
    benchmark_sid = []
    benchmark_column = DEFAULT_BENCHMARK_COLUMN
    def __init__(self):
        self.algoPeriodStartingValue = None
        self.algoPeriodEndingValue = None
        self.benchmarkPeriodStartingValue = None
        self.benchmarkPeriodEndingValue = None

        self.algoReturn = 0.0
        self.benchmarkReturn = 0.0

        # save this two for faster calculate cumulative return
        self.algoStartingValue = None
        self.benchmarkStartingValue = None

    def to_dict(self):
        fields = copy(self.__dict__)
        return fields

    def __str__(self):
        return self.to_dict().__str__()

    def __repr__(self):
        return self.to_dict().__repr__()

    @classmethod
    def setBenchmark(cls, sid, column):
        cls.benchmark_sid = sid
        cls.benchmark_column = column

    def rollover(self):
        self.algoPeriodStartingValue = self.algoPeriodEndingValue
        self.benchmarkPeriodStartingValue = self.benchmarkPeriodEndingValue
        self.algoReturn = 0.0
        self.benchmarkReturn = 0.0

    def calculateReturn(self):
        pnl = self.algoPeriodEndingValue - self.algoPeriodStartingValue
        self.algoReturn = 0.0
        if self.algoPeriodStartingValue != 0:
            self.algoReturn = pnl / self.algoPeriodStartingValue

        benchmarkPnl = self.benchmarkPeriodEndingValue - self.benchmarkPeriodStartingValue
        self.benchmarkReturn = 0.0
        if self.benchmarkPeriodStartingValue != 0:
            self.benchmarkReturn = benchmarkPnl / self.benchmarkPeriodStartingValue

    def update(self, dataEvent, context):
        if self.algoPeriodStartingValue is None:
            self.algoPeriodEndingValue = context.portfolio.value()
            self.benchmarkPeriodEndingValue = dataEvent.query(self.benchmark_sid, self.benchmark_column)
            self.algoStartingValue = context.portfolio.startingCash()
            self.benchmarkStartingValue = self.benchmarkPeriodEndingValue
            self.rollover()
            return

        self.algoPeriodEndingValue = context.portfolio.value()
        self.benchmarkPeriodEndingValue = dataEvent.query(self.benchmark_sid, self.benchmark_column)

        # take care of nan benchmark starting value
        if np.isnan(self.benchmarkStartingValue):
            self.benchmarkStartingValue = self.benchmarkPeriodEndingValue

class ReturnTracker(object):
    def __init__(self):
        self.algoReturns = deque()
        self.benchmarkReturns = deque()

        self.algoCumulativeReturns = deque()
        self.benchmarkCumulativeReturns = deque()

        self.algoMeanReturns = deque()
        self.benchmarkMeanReturns = deque()

        self.algoAnnulizedMeanReturns = deque()
        self.benchmarkAnnulizedMeanReturns = deque()

    def to_dict(self):
        fields = copy(self.__dict__)
        return fields

    def __str__(self):
        return self.to_dict().__str__()

    def __repr__(self):
        return self.to_dict().__repr__()

    '''
    calculateCumulativeReturn is deprecated. TOO SLOW
    '''
    def calculateCumulativeReturn(self, returns):
        return (1. + np.array(returns)).prod() - 1
        # return np.prod([1. + x for x in returns]) - 1

    def calculateAnnulizedReturn(self, returns):
        return KSSampleRate.getAnnulizedMultiplier(squared = False) * returns

    def update(self, period):
        '''
        period: a PeriodReturn instance
        '''
        self.algoReturns.append(period.algoReturn)
        self.benchmarkReturns.append(period.benchmarkReturn)
        numTradingUnits = len(self.algoReturns)

        '''
        self.algoCumulativeReturns.append(\
            self.calculateCumulativeReturn(self.algoReturns))
        self.benchmarkCumulativeReturns.append(\
            self.calculateCumulativeReturn(self.benchmarkReturns))
        '''
        self.algoCumulativeReturns.append(\
            (period.algoPeriodEndingValue - period.algoStartingValue) / period.algoStartingValue)
        self.benchmarkCumulativeReturns.append(\
            (period.benchmarkPeriodEndingValue - period.benchmarkStartingValue) / period.benchmarkStartingValue)

        self.algoMeanReturns.append(self.algoCumulativeReturns[-1] / numTradingUnits)
        self.benchmarkMeanReturns.append(self.benchmarkCumulativeReturns[-1] / numTradingUnits)

        self.algoAnnulizedMeanReturns.append(\
            self.calculateAnnulizedReturn(self.algoMeanReturns[-1]))
        self.benchmarkAnnulizedMeanReturns.append(\
            self.calculateAnnulizedReturn(self.benchmarkMeanReturns[-1]))

class KSPerformance(object):
    def __init__(self):
        self.period = PeriodReturn()
        self.lastdt = None
        self.nextdt = None

    def to_dict(self):
        fields = copy(self.__dict__)
        return fields

    def __repr__(self):
        return self.to_dict().__repr__()

    def __str__(self):
        return self.to_dict().__str__()

    def initialize(self):
        self.returnTracker = ReturnTracker()
        KSAnalyzer.returnTracker = self.returnTracker

    ''' deprecated methods
    def setSampleRate(self, rate):
        self.sampleRate = rate

    def setRiskless(self, riskless):
        self.riskless = riskless

    def setTradingDays(self, days):
        self.tradingDays = days

    def setTradingHours(self, hours):
        self.tradingHours = hours
    '''

    def getNextDate(self):
        if KSSampleRate.rate == KSSampleRate.MONTH:
            nextdt = self.lastdt
            current_month = nextdt.month
            while nextdt.month == current_month:
                nextdt += timedelta(days = 1)
            return nextdt
        else:
            return self.lastdt + KSSampleRate.datetimeOffset()

    '''
    def calculateMaxDrawdown(self):
        if len(self.algoCumulativeReturns) == 0:
            return self.currentDrawdown

        # The drawdown is defined as: (high - low) / high
        # The above factors out to: 1.0 - (low / high)
        drawdown = 1.0 - (1.0 + self.algoCumulativeReturns[-1])/(1.0 + self.currentMaxReturn)
        if self.currentDrawdown > drawdown:
            return self.currentDrawdown
        else:
            return drawdown

    def calculateSharpe(self):
        """
        http://en.wikipedia.org/wiki/Sharpe_ratio
        """
        assert len(self.volatility) == len(self.algoAnnulizedMeanReturns)
        return risk.sharpe_ratio(
            self.algoAnnulizedMeanReturns[-1],
            self.riskless,
            self.volatility[-1])

    def calculateSortino(self):
        """
        http://en.wikipedia.org/wiki/Sortino_ratio
        """
        assert len(self.downsideRisk) == len(self.algoAnnulizedMeanReturns)
        return risk.sortino_ratio(
            self.algoAnnulizedMeanReturns[-1],
            self.riskless,
            self.downsideRisk[-1])

    def calculateInformation(self):
        """
        http://en.wikipedia.org/wiki/Information_ratio
        """
        assert len(self.volatility) == len(self.algoAnnulizedMeanReturns)
        return risk.information_ratio(
            self.algoAnnulizedMeanReturns[-1],
            self.benchmarkAnnulizedMeanReturns[-1],
            self.volatility[-1])

    def calculateAlpha(self):
        """
        http://en.wikipedia.org/wiki/Alpha_(investment)
        """
        assert len(self.beta) == len(self.algoAnnulizedMeanReturns)
        return risk.alpha(
            self.algoAnnulizedMeanReturns[-1],
            self.riskless,
            self.benchmarkAnnulizedMeanReturns[-1],
            self.beta[-1])

    def calculateVolatility(self, returns):
        if len(returns) <= 1:
            return 0.0
        # return np.std(returns, ddof=1) * math.sqrt(252)
        return np.std(returns, ddof=1) * self.getAnnulizedMultiplier(squared = True)

    def calculateDownsideRisk(self):
        """
        https://en.wikipedia.org/wiki/Downside_risk
        """
        return risk.downside_risk(self.algoReturns,
                             self.algoMeanReturns,
                             self.getAnnulizedMultiplier())

    def calculateBeta(self):
        """
        http://en.wikipedia.org/wiki/Beta_(finance)
        """
        return risk.beta(self.algoReturns, self.benchmarkReturns)

    def calculatePerformance(self):
        # max drawdown
        self.currentMaxReturn = np.max((self.currentMaxReturn, self.algoCumulativeReturns[-1]))
        self.currentDrawdown = self.calculateMaxDrawdown()
        self.maxDrawdown.append(self.currentDrawdown)

        # volatility
        self.volatility.append(self.calculateVolatility(self.algoReturns))
        self.benchmarkVolatility.append(self.calculateVolatility(self.benchmarkReturns))

        # beta
        self.beta.append(self.calculateBeta())

        # downside risk
        self.downsideRisk.append(self.calculateDownsideRisk())

        # alpha
        self.alpha.append(self.calculateAlpha())

        # information ratio
        self.informationRatio.append(self.calculateInformation())

        # sortino ratio
        self.sortinoRatio.append(self.calculateSortino())

        # sharp ratio
        self.sharpeRatio.append(self.calculateSharpe())
    '''

    def update(self, dataEvent, context):
        self.period.update(dataEvent, context)
        # first day
        if self.lastdt is None:
            self.lastdt = dataEvent.time()
            self.nextdt = self.getNextDate()
            self.initialize()
            return

        if self.nextdt <= dataEvent.time():
            # update period return
            self.period.calculateReturn()

            # update all returns
            self.returnTracker.update(self.period)

            # rollover
            self.period.rollover()
            self.nextdt = self.getNextDate()

            


# -*- coding: utf-8 -*-

import numpy as np

class nanvar(object):
	def __init__(self, ddof=0):
		self.ddof = ddof
		self.n = 0.0
		self.mean = 0.0
		self.M2 = 0.0
		self.var = np.nan

	@property
	def value(self):
	    return self.var

	def update(self, x):
		if np.isnan(x):
			return
		self.n += 1
		delta = x - self.mean
		self.mean += delta/self.n
		self.M2 += delta*(x - self.mean)

		if self.n > self.ddof:
			self.var = self.M2 / (self.n - self.ddof)

class nanstd(nanvar):
	@property
	def value(self):
	    return np.sqrt(self.var)
	
class nancov(object):
	def __init__(self, ddof=0):
		self.ddof = ddof
		self.n = 0.0
		self.xvar = nanvar(ddof=ddof)
		self.yvar = nanvar(ddof=ddof)
		self.C2 = 0.0
		self.cov = np.full((2,2), np.nan)

	@property
	def value(self):
	    return self.cov

	def update(self, x, y):
		if not np.isnan(x) and not np.isnan(y):
			self.n += 1
			self.C2 += (self.n - 1)/(self.n) * (x - self.xvar.mean) * (y -self.yvar.mean)
			if self.n > self.ddof:
				self.cov[0,1] = self.cov[1,0] = self.C2 / (self.n - self.ddof)
			self.xvar.update(x)
			self.yvar.update(y)
			self.cov[0,0] = self.xvar.value
			self.cov[1,1] = self.yvar.value



# -*- coding: utf-8 -*-

from six import with_metaclass
import abc
import sys
import numpy as np
import math
from datetime import datetime, timedelta
from copy import copy

from keystone.performance import risk
from keystone.performance.sample_rate import KSSampleRate
from keystone.riskless import Riskless
from keystone.utils import numpy_utils


class KSAnalyzer(with_metaclass(abc.ABCMeta)):

    '''
    Abstract class of analyzer.

    Class properties:
    returnTracker: ReturnTracker instance, shared by all analyzer
    '''
    returnTracker = None

    def getAnnulizedMultiplier(self, squared=True):
        return KSSampleRate.getAnnulizedMultiplier(squared=squared)

    def update(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def value(self, *args, **kwargs):
        pass


class KSReturn(KSAnalyzer):

    @property
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.algoCumulativeReturns) <= 1:
            return 0.0
        return self.returnTracker.algoCumulativeReturns[-1]


class KSBenchmarkReturn(KSAnalyzer):

    @property
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.benchmarkCumulativeReturns) <= 1:
            return 0.0
        return self.returnTracker.benchmarkCumulativeReturns[-1]


class KSVolatility(KSAnalyzer):

    def __init__(self):
        self.std = numpy_utils.nanstd(ddof=1)

    @property
    def value(self, *args, **kwargs):
        return self.std.value * self.getAnnulizedMultiplier(squared=True)

    def update(self, *args, **kwargs):
        if self.returnTracker is not None and len(self.returnTracker.algoReturns) > 0:
            self.std.update(self.returnTracker.algoReturns[-1])


class KSBeta(KSAnalyzer):

    """
    http://en.wikipedia.org/wiki/Beta_(finance)
    """

    def __init__(self):
        self.cov = numpy_utils.nancov(ddof=1)

    @property
    def value(self, *args, **kwargs):
        algorithm_covariance = self.cov.value[0, 1]
        benchmark_variance = self.cov.value[1, 1]
        return algorithm_covariance / benchmark_variance

    def update(self, *args, **kwargs):
        if self.returnTracker is not None and len(self.returnTracker.algoReturns) > 0:
            self.cov.update(
                self.returnTracker.algoReturns[-1], self.returnTracker.benchmarkReturns[-1])


class KSDownsideRisk(KSAnalyzer):

    """
    https://en.wikipedia.org/wiki/Downside_risk
    """

    def __init__(self):
        self.std = numpy_utils.nanstd(ddof=1)

    @property
    def value(self, *args, **kwargs):
        return self.std.value * self.getAnnulizedMultiplier(squared=True)

    def update(self, *args, **kwargs):
        if self.returnTracker is not None and len(self.returnTracker.algoReturns) > 0:
            rets = np.round(self.returnTracker.algoReturns[-1], 8)
            mar = np.round(self.returnTracker.algoMeanReturns[-1], 8)
            if rets < mar:
                self.std.update(rets - mar)

class KSAlpha(KSAnalyzer):

    """
    http://en.wikipedia.org/wiki/Alpha_(investment)
    """

    def __init__(self):
        self.beta = KSBeta()

    @property
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.algoReturns) < 1:
            return np.nan
        return risk.alpha(
            self.returnTracker.algoAnnulizedMeanReturns[-1],
            Riskless.value,
            self.returnTracker.benchmarkAnnulizedMeanReturns[-1],
            self.beta.value)

    def update(self, *args, **kwargs):
        self.beta.update(args, kwargs)

class KSInformationRatio(KSAnalyzer):
    """
    http://en.wikipedia.org/wiki/Information_ratio
    """

    def __init__(self):
        self.volatility = KSVolatility()

    @property
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.algoReturns) <= 1:
            return np.nan

        return risk.information_ratio(
            self.returnTracker.algoAnnulizedMeanReturns[-1],
            self.returnTracker.benchmarkAnnulizedMeanReturns[-1],
            self.volatility.value)

    def update(self, *args, **kwargs):
        self.volatility.update(args, kwargs)

class KSSortinoRatio(KSAnalyzer):
    """
    http://en.wikipedia.org/wiki/Sortino_ratio
    """
    def __init__(self):
        self.downsideRisk = KSDownsideRisk()

    @property
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.algoReturns) <= 1:
            return np.nan

        return risk.sortino_ratio(
            self.returnTracker.algoAnnulizedMeanReturns[-1],
            Riskless.value,
            self.downsideRisk.value)

    def update(self, *args, **kwargs):
        self.downsideRisk.update(args, kwargs)


class KSSharpRatio(KSAnalyzer):
    """
    http://en.wikipedia.org/wiki/Sharpe_ratio
    """
    def __init__(self):
        self.volatility = KSVolatility()

    @property
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.algoReturns) <= 1:
            return np.nan

        return risk.sharpe_ratio(
            self.returnTracker.algoAnnulizedMeanReturns[-1],
            Riskless.value,
            self.volatility.value)

    def update(self, *args, **kwargs):
        self.volatility.update(args, kwargs)

class KSMaxDrawdown(KSAnalyzer):

    def __init__(self):
        KSAnalyzer.__init__(self)
        self.currentMaxReturn = -np.inf
        self.currentDrawdown = -np.inf

    @property
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.algoCumulativeReturns) == 0:
            return 0.0

        self.currentMaxReturn = np.nanmax(
            (self.currentMaxReturn, self.returnTracker.algoCumulativeReturns[-1]))
        # The drawdown is defined as: (high - low) / high
        # The above factors out to: 1.0 - (low / high)
        drawdown = 1.0 - \
            (1.0 +
             self.returnTracker.algoCumulativeReturns[-1]) / (1.0 + self.currentMaxReturn)
        if self.currentDrawdown < drawdown:
            self.currentDrawdown = drawdown

        return self.currentDrawdown


class KSDefaultAnalyzer(KSAnalyzer):

    def __init__(self):
        KSAnalyzer.__init__(self)
        self.benchmark = KSBenchmarkReturn()
        self.returns = KSReturn()
        self.alpha = KSAlpha()
        self.beta = KSBeta()
        self.volatility = KSVolatility()
        self.maxDrawdown = KSMaxDrawdown()
        self.downsideRisk = KSDownsideRisk()
        self.sharpeRatio = KSSharpRatio()
        self.sortinoRatio = KSSortinoRatio()
        self.informationRatio = KSInformationRatio()

    def update(self, *args, **kwargs):
        self.alpha.update(args, kwargs)
        self.beta.update(args, kwargs)
        self.volatility.update(args, kwargs)
        self.downsideRisk.update(args, kwargs)
        self.sharpeRatio.update(args, kwargs)
        self.sortinoRatio.update(args, kwargs)
        self.informationRatio.update(args, kwargs)

    def value(self, *args, **kwargs):
        ret = {
            'return': self.returns.value,
            'benchmark': self.benchmark.value,
            'alpha': self.alpha.value,
            'beta': self.beta.value,
            'volatility': self.volatility.value,
            'max_drawdown': self.maxDrawdown.value,
            'downside_risk': self.downsideRisk.value,
            'sharpe_ratio': self.sharpeRatio.value,
            'sortino_ratio': self.sortinoRatio.value,
            'information_ratio': self.informationRatio.value
        }
        return ret


class KSCumulativeAnalyzer(KSAnalyzer):

    def __init__(self):
        self.returns = []
        self.benchmark = []
        self.alpha = []
        self.beta = []
        self.volatility = []
        self.max_drawdown = []
        self.downside_risk = []
        self.sharpe_ratio = []
        self.sortino_ratio = []
        self.information_ratio = []

        self.count = 0
        self.defaultAnalyzer = KSDefaultAnalyzer()

    def value(self, *args, **kwargs):
        fields = copy(self.__dict__)
        return fields

    def update(self, *args, **kwargs):
        self.count += 1
        if self.count > 2:
            self.returns.append(self.defaultAnalyzer.returns.value)
            self.benchmark.append(self.defaultAnalyzer.benchmark.value)
            self.alpha.append(self.defaultAnalyzer.alpha.value)
            self.beta.append(self.defaultAnalyzer.beta.value)
            self.volatility.append(self.defaultAnalyzer.volatility.value)
            self.max_drawdown.append(self.defaultAnalyzer.maxDrawdown.value)
            self.downside_risk.append(
                self.defaultAnalyzer.downsideRisk.value)
            self.sharpe_ratio.append(self.defaultAnalyzer.sharpeRatio.value)
            self.sortino_ratio.append(
                self.defaultAnalyzer.sortinoRatio.value)
            self.information_ratio.append(
                self.defaultAnalyzer.informationRatio.value)



# -*- coding: utf-8 -*-

from __future__ import print_function
import time
import pandas as pd
import sys
from six import with_metaclass
import abc

from keystone.api import keystone_class, api_method
from keystone.engine import KSEventEngine

class KSObserver(with_metaclass(abc.ABCMeta)):
    
    @abc.abstractmethod
    def onDataEvent(self, dataEvent):
        pass
    
    def onOrderEvent(self, orderEvent):
        pass
    

class KSCoordinator(object):
    '''
    Global Event Dispatcher
    '''
    
    # pass pd.Series function handler to KSEventEngine,
    # enable cpp interface KSDataEvent.query call pd.Series
    # function and return a pd.Series object 

    # self.eventEngine = KSEventEngine(pd.DataFrame)
    eventEngine = KSEventEngine(pd.Series)
    dataObservers = []
    orderObservers = []
    
    @classmethod
    def addSource(cls, source):
        cls.eventEngine.addSource(source)

    @classmethod
    def addDataObserver(cls, observer):
        cls.dataObservers.append(observer)
    
    @classmethod
    def addOrderObserver(cls, observer):
        cls.orderObservers.append(observer)
        
    @classmethod
    def dispatchDataEvent(cls, dataEvent):
        for observer in cls.dataObservers:
            observer.onDataEvent(dataEvent)
    
    @classmethod 
    def dispatchOrderEvent(cls, orderEvent):
        for observer in cls.orderObservers:
            observer.onOrderEvent(orderEvent)
    
    @classmethod
    def run(cls):
        cls.eventEngine.run();
        while 1:
            dataEvent = cls.eventEngine.getNextEvent()
            if dataEvent is None:
                break
            else:
                cls.dispatchDataEvent(dataEvent)
            


# -*- coding: utf-8 -*-

from __future__ import print_function
import json
import abc
import pandas as pd
from datetime import datetime
from six import with_metaclass

DATAFRAME_KEY = "dataframe"

class KSTableFile(object):
	def __init__(self, path):
		self.path = path

	def read_dataframe(self, key=DATAFRAME_KEY):
		return pd.read_hdf(self.path, key)

	def write_dataframe(self, dataframe, key=DATAFRAME_KEY, mode='w'):
		if not isinstance(dataframe, pd.DataFrame):
			raise TypeError("KSTableFile::write_dataframe(): dataframe MUST be an instance of DataFrame")
		dataframe.to_hdf(self.path, key, mode=mode)


# -*- coding: utf-8 -*-

from __future__ import print_function
from collections import deque, Iterable
import pandas as pd

from keystone.utils import math_utils
from keystone.api import keystone_class, api_method
from keystone.coordinator import KSObserver
from keystone.py3compat import PY3, builtin_mod, iteritems, unicode_type, string_types
import datetime


class KSHistory(KSObserver):
    def __init__(self, capacity):
        if not math_utils.isint(capacity) or capacity == 0:
            raise TypeError("capacity格式错误，capacity必须为非0整数。")
        self.__capacity = capacity
        self.__data = deque([],capacity)
        
    
    def query(self, startTime, endTime, ids, fieldname):
        # print "[startTime,endTime] = [" + str(startTime) + "," + str(endTime) +"]"
        # Check parameters
        if not isinstance(startTime, datetime.datetime):
            raise TypeError("startTime格式错误，startTime必须为datetime类型。")
        
        if not isinstance(endTime, datetime.datetime):
            raise TypeError("endTime格式错误，endTime必须为datetime类型。")
        
        if not isinstance(fieldname, string_types):
            raise TypeError("fieldname格式错误, fieldname必须是字符串。")
        
        if not isinstance(ids, string_types) and not isinstance(ids, Iterable):
            raise TypeError("securities格式错误, securities必须是字符串或字符串数组。")
        
        if isinstance(ids, Iterable):
            for x in ids:
                if not isinstance(x, string_types):
                        raise TypeError("securities格式错误, securities必须是字符串或字符串数组。")
        
        # Query
        ret = pd.DataFrame()
        for dataEvent in self.__data:
            dt = dataEvent.time()
            if startTime <= dt and dt <= endTime:
                ret = ret.join(dataEvent.query(ids, fieldname), how = 'outer')
                
        return ret
    
    
    def queryN(self, n, ids, fieldname):
        # Check parameters
        if not math_utils.isint(n) or n == 0:
            raise TypeError("n格式错误，n必须为非0整数。")
        
        if not isinstance(fieldname, string_types):
            raise TypeError("fieldname格式错误, fieldname必须是字符串。")
        
        if not isinstance(ids, string_types) and not isinstance(ids, Iterable):
            raise TypeError("securities格式错误, securities必须是字符串或字符串数组。")
        
        if isinstance(ids, Iterable):
            for x in ids:
                if not isinstance(x, string_types):
                        raise TypeError("securities格式错误, securities必须是字符串或字符串数组。")
                    
        # QueryN
        ret = pd.DataFrame()
        m = min(n, len(self.__data))
        iter = range(len(self.__data) - m, len(self.__data))
        for i in iter:
            ret = ret.join(self.__data[i].query(ids, fieldname), how = 'outer')
            
        return ret
    
    def onDataEvent(self, dataEvent):
        self.__data.append(dataEvent)
    


# -*- coding: utf-8 -*-

import zmq
import sys

def create_publisher_socket(publisher_url, syn_url = None, SUBSCRIBERS_EXPECTED = 1):
	context = zmq.Context()

	# Socket to talk to clients
	publisher = context.socket(zmq.PUB)
	# set SNDHWM, so we don't drop messages for slow subscribers
	# publisher.sndhwm = 1100000
	publisher.bind(publisher_url)

	if syn_url is not None:
		# Socket to receive signals
		syncservice = context.socket(zmq.REP)
		syncservice.bind(syn_url)

		# Get synchronization from subscribers
		subscribers = 0
		while subscribers < SUBSCRIBERS_EXPECTED:
		    # wait for synchronization request
		    msg = syncservice.recv()
		    # send synchronization reply
		    syncservice.send(b'')
		    subscribers += 1
		    # print >>sys.stderr,("+1 subscriber (%i/%i)" % (subscribers, SUBSCRIBERS_EXPECTED))

		# Now broadcast exactly 1M updates followed by END
		# for i in range(1000000):
		#     publisher.send(b'Rhubarb')

		# publisher.send(b'END')
		syncservice.close()
	return publisher


# -*- coding: utf-8 -*-

from keystone.strategy import KSStrategy, KSStrategyRunner
from keystone.sources import KSCsvSource
from keystone.performance.analyzer import KSDefaultAnalyzer, KSCumulativeAnalyzer, KSSampleRate

import sys
import pandas as pd
from datetime import datetime, timedelta
import time
# sys.platform.startswith('win'):
class MyStrategy(KSStrategy):
    def __init__(self):
        self.cash = 100000

    def onData(self, data, context, action):
        #print context.universe._KSUniverse__price
        #print data.securities()

        # print data.getAllData()
        # raw_input()
        # return
        print(u"在onData中。")
        securities = data.securities()
        print(securities)
        latestPrice = data.query([], 'price')
        print(latestPrice)
        print(data.time())

        print(latestPrice)
        print(context.history.queryN(3,['002035', '002010','002210'], 'price'))
        print(context.history.query(data.time()-timedelta(3), data.time(),['002035', '002010','002210'], 'price'))
        print(context.portfolio.cash())
        print(context.portfolio.securities())

        if not context.portfolio.hasPosition('002035'):
            orderid = action.order('002035',100)
            #print "orderid: " + str(orderid)
            order = action.getOrder(orderid)
            #print "order is:"
            #print order
        else:
            orderid = action.orderValue('002035',0)
            #print "orderid: " + str(orderid)
            order = action.getOrder(orderid)
            #print "order is:"
            #print order

        if context.portfolio.hasPosition('002035'):
            print(context.portfolio.getPosition('002035'))
        print(context.portfolio.value())
        print(context.portfolio.cash())
        print(context.analyzers[0].returns)
        print(context.analyzers[0].value())
        print("=========================")
        # time.sleep(2)
        # c=raw_input()

    def onOrderEvent(self, orderEvent):
        print("recieve order update info")
        #print orderEvent

def run():
    c = KSCsvSource('../../data/market.csv', 'dt', '%Y-%m-%d %H:%M:%S')
    c.tagAsPriceData('sid','price')
    c.setDelimiter(',')
    signal = KSCsvSource('../../data/signal.csv', 'dt', '%Y-%m-%d %H:%M:%S')
    analyzer = KSDefaultAnalyzer()
    cumulative_analyzer = KSCumulativeAnalyzer()

    runner = KSStrategyRunner()
    runner.setHistoryCapacity(10)
    runner.setCash(100000)
    runner.addSource(c)
    runner.addSource(signal)
    runner.setStrategy(MyStrategy)
    runner.setBenchmark('price', '002035')
    # runner.setSampleRate(KSSampleRate.DAY)
    runner.setInstantMatch()
    runner.attachAnalyzer(analyzer)
    # runner.attachAnalyzer(cumulative_analyzer)
    # runner.setStartTime(datetime.strptime("2014-05-05 00:00:00", "%Y-%m-%d %H:%M:%S"))
    # runner.setEndTime(datetime.strptime("2014-05-08 00:00:00", "%Y-%m-%d %H:%M:%S"))
    runner.setStopPercentage(1.0)
    runner.run()

if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-

from __future__ import print_function
import json
import abc
import pandas as pd
from datetime import datetime
from six import with_metaclass

class AppUniverse(object):
	def __init__(self, path):
		self.path = path
		self.
class KSDataEvent(object):
	data = []
	def __init__(self, path):
		pass

	@classmethod
	def addSource(cls, path, config):


# -*- coding: utf-8 -*-

class KSEndTimeReachedException(Exception):
    def __str__(self):
        return "Reach End Time"
    
class KSStopLineReachedException(Exception):
    def __str__(self):
        return "Reach Stop Line"
    


# -*- coding: utf-8 -*-

"""Utilities to manipulate JSON objects."""

import math
import re
import types
from datetime import datetime
import numpy as np
import numbers
numbers.Integral.register(np.integer)

try:
    # base64.encodestring is deprecated in Python 3.x
    from base64 import encodebytes
except ImportError:
    # Python 2.x
    from base64 import encodestring as encodebytes

from keystone import py3compat
from keystone.py3compat import PY3, builtin_mod, iteritems, unicode_type, string_types, DEFAULT_ENCODING
next_attr_name = '__next__' if py3compat.PY3 else 'next'

#-----------------------------------------------------------------------------
# Globals and constants
#-----------------------------------------------------------------------------
# timestamp formats
UTC_TIMESTAMP = 'TIMESTAMP_SECOND'
UTC_TIMESTAMP_MILLISECOND = 'TIMESTAMP_MILLISECOND'
ISO8601 = "%Y-%m-%dT%H:%M:%S.%f"
ISO8601_PAT=re.compile(r"^(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2})(\.\d{1,6})?Z?([\+\-]\d{2}:?\d{2})?$")

# holy crap, strptime is not threadsafe.
# Calling it once at import seems to help.
datetime.strptime("1", "%d")

#-----------------------------------------------------------------------------
# Classes and functions
#-----------------------------------------------------------------------------


def parse_date(s):
    """parse an ISO8601 date string

    If it is None or not a valid ISO8601 timestamp,
    it will be returned unmodified.
    Otherwise, it will return a datetime object.
    """
    if s is None:
        return s
    m = ISO8601_PAT.match(s)
    if m:
        # FIXME: add actual timezone support
        # this just drops the timezone info
        notz, ms, tz = m.groups()
        if not ms:
            ms = '.0'
        notz = notz + ms
        return datetime.strptime(notz, ISO8601)
    return s


def extract_dates(obj):
    """extract ISO8601 dates from unpacked JSON"""
    if isinstance(obj, dict):
        new_obj = {}  # don't clobber
        for k, v in iteritems(obj):
            new_obj[k] = extract_dates(v)
        obj = new_obj
    elif isinstance(obj, (list, tuple)):
        obj = [extract_dates(o) for o in obj]
    elif isinstance(obj, string_types):
        obj = parse_date(obj)
    return obj


def squash_dates(obj):
    """squash datetime objects into ISO8601 strings"""
    if isinstance(obj, dict):
        obj = dict(obj)  # don't clobber
        for k, v in iteritems(obj):
            obj[k] = squash_dates(v)
    elif isinstance(obj, (list, tuple)):
        obj = [squash_dates(o) for o in obj]
    elif isinstance(obj, datetime):
        obj = obj.isoformat()
    return obj


def date_default(obj):
    """default function for packing datetime objects in JSON."""
    if isinstance(obj, datetime):
        return obj.isoformat()
    else:
        raise TypeError("%r is not JSON serializable" % obj)



# constants for identifying png/jpeg data
PNG = b'\x89PNG\r\n\x1a\n'
# front of PNG base64-encoded
PNG64 = b'iVBORw0KG'
JPEG = b'\xff\xd8'
# front of JPEG base64-encoded
JPEG64 = b'/9'
# front of PDF base64-encoded
PDF64 = b'JVBER'

def encode_images(format_dict):
    """b64-encodes images in a displaypub format dict

    Perhaps this should be handled in json_clean itself?

    Parameters
    ----------

    format_dict : dict
        A dictionary of display data keyed by mime-type

    Returns
    -------

    format_dict : dict
        A copy of the same dictionary,
        but binary image data ('image/png', 'image/jpeg' or 'application/pdf')
        is base64-encoded.

    """
    encoded = format_dict.copy()

    pngdata = format_dict.get('image/png')
    if isinstance(pngdata, bytes):
        # make sure we don't double-encode
        if not pngdata.startswith(PNG64):
            pngdata = encodebytes(pngdata)
        encoded['image/png'] = pngdata.decode('ascii')

    jpegdata = format_dict.get('image/jpeg')
    if isinstance(jpegdata, bytes):
        # make sure we don't double-encode
        if not jpegdata.startswith(JPEG64):
            jpegdata = encodebytes(jpegdata)
        encoded['image/jpeg'] = jpegdata.decode('ascii')

    pdfdata = format_dict.get('application/pdf')
    if isinstance(pdfdata, bytes):
        # make sure we don't double-encode
        if not pdfdata.startswith(PDF64):
            pdfdata = encodebytes(pdfdata)
        encoded['application/pdf'] = pdfdata.decode('ascii')

    return encoded


def json_clean(obj, datetime_format = ISO8601):
    """Clean an object to ensure it's safe to encode in JSON.

    Atomic, immutable objects are returned unmodified.  Sets and tuples are
    converted to lists, lists are copied and dicts are also copied.

    Note: dicts whose keys could cause collisions upon encoding (such as a dict
    with both the number 1 and the string '1' as keys) will cause a ValueError
    to be raised.

    Parameters
    ----------
    obj : any python object

    Returns
    -------
    out : object

      A version of the input which will not cause an encoding error when
      encoded as JSON.  Note that this function does not *encode* its inputs,
      it simply sanitizes it so that there will be no encoding errors later.

    """
    # types that are 'atomic' and ok in json as-is.
    atomic_ok = (unicode_type, type(None))

    # containers that we need to convert into lists
    container_to_list = (tuple, set, types.GeneratorType)

    if isinstance(obj, float):
        # cast out-of-range floats to their reprs
        if math.isnan(obj) or math.isinf(obj):
            return repr(obj)
        return float(obj)
    
    # if isinstance(obj, int):
    if isinstance(obj, numbers.Integral):
        # cast int to int, in case subclasses override __str__ (e.g. boost enum, #4598)
        if isinstance(obj, bool):
            # bools are ints, but we don't want to cast them to 0,1
            return obj
        return int(obj)

    if isinstance(obj, atomic_ok):
        return obj

    if isinstance(obj, bytes):
        return obj.decode(DEFAULT_ENCODING, 'replace')

    if isinstance(obj, container_to_list) or (
        hasattr(obj, '__iter__') and hasattr(obj, next_attr_name)):
        obj = list(obj)

    if isinstance(obj, list):
        return [json_clean(x, datetime_format) for x in obj]

    if isinstance(obj, dict):
        # First, validate that the dict won't lose data in conversion due to
        # key collisions after stringification.  This can happen with keys like
        # True and 'true' or 1 and '1', which collide in JSON.
        nkeys = len(obj)
        nkeys_collapsed = len(set(map(unicode_type, obj)))
        if nkeys != nkeys_collapsed:
            raise ValueError('dict cannot be safely converted to JSON: '
                             'key collision would lead to dropped values')
        # If all OK, proceed by making the new dict that will be json-safe
        out = {}
        for k,v in iteritems(obj):
            out[unicode_type(k)] = json_clean(v, datetime_format)
        return out
    if isinstance(obj, datetime):
        if datetime_format == UTC_TIMESTAMP:
            return (obj - datetime(1970,1,1)).total_seconds()
        elif datetime_format == UTC_TIMESTAMP_MILLISECOND:
            return (obj - datetime(1970,1,1)).total_seconds()*1000.0
        else:
            return obj.strftime(datetime_format)
    
    # we don't understand it, it's probably an unserializable object
    raise ValueError("Can't clean for JSON: %r(%s)" %(obj, type(obj)))


# -*- coding: utf-8 -*-

from __future__ import print_function
from six import with_metaclass
import abc
import numpy as np
from datetime import datetime

from keystone.api import keystone_class, api_method
from keystone.utils import math_utils

class KSSlippageModel(with_metaclass(abc.ABCMeta)):
    def __init__(self):
        pass
    
    @abc.abstractmethod
    def compute(self, dataEvent, order, *args, **kwargs):
        '''
        INPUT:
            dataEvent - Current data event
            order - KSOrder instance about the current order
            args - User custom args
            kwargs - User custom kwargs
        OUTPUT:
            user MUST return a deal price
        '''
        return 0
    
class KSFixedSlippageModel(KSSlippageModel):
    def __init__(self, rate):
        KSSlippageModel.__init__(self)
        if not math_utils.isnumber(rate) or rate < 0 or rate > 1:
            raise TypeError("rate必须为数字，且 0 <= rate <= 1。")
        self.spread = rate
    
    def compute(self, dataEvent, order, *args, **kwargs):
        sid = order.securityId()
        shares = order.remaining()
        dealPrice = dataEvent.query(sid, 'price')
        direction = np.sign(shares)

        return dealPrice * (1 + self.spread / 2 * direction)
